# coding: utf-8

"""
    Webhook Type Definition

    Webhook event definition of the LINE Messaging API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class EventMode(str, Enum):
    """
    Channel state.
    """

    """
    allowed enum values
    """
    ACTIVE = 'active'
    STANDBY = 'standby'

    @classmethod
    def from_json(cls, json_str: str) -> EventMode:
        """Create an instance of EventMode from a JSON string"""
        return EventMode(json.loads(json_str))





# coding: utf-8

"""
    Channel Access Token API

    This document describes Channel Access Token API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic.v1 import BaseModel, Field, StrictInt, StrictStr

class IssueStatelessChannelAccessTokenResponse(BaseModel):
    """
    Issued stateless channel access token
    https://developers.line.biz/en/reference/messaging-api/#issue-stateless-channel-access-token
    """
    access_token: StrictStr = Field(..., description="A stateless channel access token. The token is an opaque string which means its format is an implementation detail and the consumer of this token should never try to use the data parsed from the token. ")
    expires_in: StrictInt = Field(..., description="Duration in seconds after which the issued access token expires")
    token_type: StrictStr = Field(..., description="Token type. The value is always `Bearer`.")

    __properties = ["access_token", "expires_in", "token_type"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> IssueStatelessChannelAccessTokenResponse:
        """Create an instance of IssueStatelessChannelAccessTokenResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> IssueStatelessChannelAccessTokenResponse:
        """Create an instance of IssueStatelessChannelAccessTokenResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return IssueStatelessChannelAccessTokenResponse.parse_obj(obj)

        _obj = IssueStatelessChannelAccessTokenResponse.parse_obj({
            "access_token": obj.get("access_token"),
            "expires_in": obj.get("expires_in"),
            "token_type": obj.get("token_type") if obj.get("token_type") is not None else 'Bearer'
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API(Insight)

    This document describes LINE Messaging API(Insight).  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictInt

class GetStatisticsPerUnitResponseOverview(BaseModel):
    """
    Statistics related to messages.
    https://developers.line.biz/en/reference/messaging-api/#get-statistics-per-unit-response
    """
    unique_impression: Optional[StrictInt] = Field(None, alias="uniqueImpression", description="Number of users who opened the message, meaning they displayed at least 1 bubble.")
    unique_click: Optional[StrictInt] = Field(None, alias="uniqueClick", description="Number of users who opened any URL in the message.")
    unique_media_played: Optional[StrictInt] = Field(None, alias="uniqueMediaPlayed", description="Number of users who started playing any video or audio in the message.")
    unique_media_played100_percent: Optional[StrictInt] = Field(None, alias="uniqueMediaPlayed100Percent", description="Number of users who played the entirety of any video or audio in the message.")

    __properties = ["uniqueImpression", "uniqueClick", "uniqueMediaPlayed", "uniqueMediaPlayed100Percent"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GetStatisticsPerUnitResponseOverview:
        """Create an instance of GetStatisticsPerUnitResponseOverview from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if unique_impression (nullable) is None
        # and __fields_set__ contains the field
        if self.unique_impression is None and "unique_impression" in self.__fields_set__:
            _dict['uniqueImpression'] = None

        # set to None if unique_click (nullable) is None
        # and __fields_set__ contains the field
        if self.unique_click is None and "unique_click" in self.__fields_set__:
            _dict['uniqueClick'] = None

        # set to None if unique_media_played (nullable) is None
        # and __fields_set__ contains the field
        if self.unique_media_played is None and "unique_media_played" in self.__fields_set__:
            _dict['uniqueMediaPlayed'] = None

        # set to None if unique_media_played100_percent (nullable) is None
        # and __fields_set__ contains the field
        if self.unique_media_played100_percent is None and "unique_media_played100_percent" in self.__fields_set__:
            _dict['uniqueMediaPlayed100Percent'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> GetStatisticsPerUnitResponseOverview:
        """Create an instance of GetStatisticsPerUnitResponseOverview from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return GetStatisticsPerUnitResponseOverview.parse_obj(obj)

        _obj = GetStatisticsPerUnitResponseOverview.parse_obj({
            "unique_impression": obj.get("uniqueImpression"),
            "unique_click": obj.get("uniqueClick"),
            "unique_media_played": obj.get("uniqueMediaPlayed"),
            "unique_media_played100_percent": obj.get("uniqueMediaPlayed100Percent")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import StrictStr, validator
from linebot.v3.messaging.models.action import Action
from linebot.v3.messaging.models.flex_box import FlexBox
from linebot.v3.messaging.models.flex_bubble_styles import FlexBubbleStyles
from linebot.v3.messaging.models.flex_component import FlexComponent
from linebot.v3.messaging.models.flex_container import FlexContainer

class FlexBubble(FlexContainer):
    """
    FlexBubble
    """
    direction: Optional[StrictStr] = None
    styles: Optional[FlexBubbleStyles] = None
    header: Optional[FlexBox] = None
    hero: Optional[FlexComponent] = None
    body: Optional[FlexBox] = None
    footer: Optional[FlexBox] = None
    size: Optional[StrictStr] = None
    action: Optional[Action] = None
    type: str = "bubble"

    __properties = ["type", "direction", "styles", "header", "hero", "body", "footer", "size", "action"]

    @validator('direction')
    def direction_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('ltr', 'rtl'):
            raise ValueError("must be one of enum values ('ltr', 'rtl')")
        return value

    @validator('size')
    def size_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('nano', 'micro', 'deca', 'hecto', 'kilo', 'mega', 'giga'):
            raise ValueError("must be one of enum values ('nano', 'micro', 'deca', 'hecto', 'kilo', 'mega', 'giga')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> FlexBubble:
        """Create an instance of FlexBubble from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of styles
        if self.styles:
            _dict['styles'] = self.styles.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of header
        if self.header:
            _dict['header'] = self.header.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of hero
        if self.hero:
            _dict['hero'] = self.hero.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of body
        if self.body:
            _dict['body'] = self.body.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of footer
        if self.footer:
            _dict['footer'] = self.footer.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of action
        if self.action:
            _dict['action'] = self.action.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> FlexBubble:
        """Create an instance of FlexBubble from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return FlexBubble.parse_obj(obj)

        _obj = FlexBubble.parse_obj({
            "type": obj.get("type"),
            "direction": obj.get("direction"),
            "styles": FlexBubbleStyles.from_dict(obj.get("styles")) if obj.get("styles") is not None else None,
            "header": FlexBox.from_dict(obj.get("header")) if obj.get("header") is not None else None,
            "hero": FlexComponent.from_dict(obj.get("hero")) if obj.get("hero") is not None else None,
            "body": FlexBox.from_dict(obj.get("body")) if obj.get("body") is not None else None,
            "footer": FlexBox.from_dict(obj.get("footer")) if obj.get("footer") is not None else None,
            "size": obj.get("size"),
            "action": Action.from_dict(obj.get("action")) if obj.get("action") is not None else None
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, constr

class UpdateAudienceGroupDescriptionRequest(BaseModel):
    """
    Rename an audience
    https://developers.line.biz/en/reference/messaging-api/#set-description-audience-group
    """
    description: Optional[constr(strict=True, max_length=120, min_length=1)] = Field(None, description="The audience's name. This is case-insensitive, meaning AUDIENCE and audience are considered identical. Max character limit: 120 ")

    __properties = ["description"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> UpdateAudienceGroupDescriptionRequest:
        """Create an instance of UpdateAudienceGroupDescriptionRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> UpdateAudienceGroupDescriptionRequest:
        """Create an instance of UpdateAudienceGroupDescriptionRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return UpdateAudienceGroupDescriptionRequest.parse_obj(obj)

        _obj = UpdateAudienceGroupDescriptionRequest.parse_obj({
            "description": obj.get("description")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import BaseModel, Field, StrictBool, StrictStr, conlist, constr
from linebot.v3.audience.models.audience import Audience

class CreateAudienceGroupRequest(BaseModel):
    """
    Create audience for uploading user IDs (by JSON)
    https://developers.line.biz/en/reference/messaging-api/#create-upload-audience-group
    """
    description: Optional[constr(strict=True, max_length=120)] = Field(None, description="The audience's name. This is case-insensitive, meaning AUDIENCE and audience are considered identical. Max character limit: 120 ")
    is_ifa_audience: Optional[StrictBool] = Field(None, alias="isIfaAudience", description="To specify recipients by IFAs: set true. To specify recipients by user IDs: set false or omit isIfaAudience property. ")
    upload_description: Optional[StrictStr] = Field(None, alias="uploadDescription", description="The description to register for the job (in jobs[].description). ")
    audiences: Optional[conlist(Audience, max_items=10000)] = Field(None, description="An array of user IDs or IFAs. Max number: 10,000 ")

    __properties = ["description", "isIfaAudience", "uploadDescription", "audiences"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateAudienceGroupRequest:
        """Create an instance of CreateAudienceGroupRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in audiences (list)
        _items = []
        if self.audiences:
            for _item in self.audiences:
                if _item:
                    _items.append(_item.to_dict())
            _dict['audiences'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateAudienceGroupRequest:
        """Create an instance of CreateAudienceGroupRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateAudienceGroupRequest.parse_obj(obj)

        _obj = CreateAudienceGroupRequest.parse_obj({
            "description": obj.get("description"),
            "is_ifa_audience": obj.get("isIfaAudience"),
            "upload_description": obj.get("uploadDescription"),
            "audiences": [Audience.from_dict(_item) for _item in obj.get("audiences")] if obj.get("audiences") is not None else None
        })
        return _obj




# coding: utf-8

# flake8: noqa

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import apis into sdk package
from linebot.v3.audience.api.manage_audience import ManageAudience
from linebot.v3.audience.api.manage_audience_blob import ManageAudienceBlob

from linebot.v3.audience.api.async_manage_audience import AsyncManageAudience
from linebot.v3.audience.api.async_manage_audience_blob import AsyncManageAudienceBlob


# import ApiClient
from linebot.v3.audience.api_response import ApiResponse
from linebot.v3.audience.api_client import ApiClient
from linebot.v3.audience.async_api_client import AsyncApiClient
from linebot.v3.audience.configuration import Configuration
from linebot.v3.audience.exceptions import OpenApiException
from linebot.v3.audience.exceptions import ApiTypeError
from linebot.v3.audience.exceptions import ApiValueError
from linebot.v3.audience.exceptions import ApiKeyError
from linebot.v3.audience.exceptions import ApiAttributeError
from linebot.v3.audience.exceptions import ApiException

# import models into sdk package
from linebot.v3.audience.models.adaccount import Adaccount
from linebot.v3.audience.models.add_audience_to_audience_group_request import AddAudienceToAudienceGroupRequest
from linebot.v3.audience.models.audience import Audience
from linebot.v3.audience.models.audience_group import AudienceGroup
from linebot.v3.audience.models.audience_group_create_route import AudienceGroupCreateRoute
from linebot.v3.audience.models.audience_group_failed_type import AudienceGroupFailedType
from linebot.v3.audience.models.audience_group_job import AudienceGroupJob
from linebot.v3.audience.models.audience_group_job_failed_type import AudienceGroupJobFailedType
from linebot.v3.audience.models.audience_group_job_status import AudienceGroupJobStatus
from linebot.v3.audience.models.audience_group_job_type import AudienceGroupJobType
from linebot.v3.audience.models.audience_group_permission import AudienceGroupPermission
from linebot.v3.audience.models.audience_group_status import AudienceGroupStatus
from linebot.v3.audience.models.audience_group_type import AudienceGroupType
from linebot.v3.audience.models.create_audience_group_request import CreateAudienceGroupRequest
from linebot.v3.audience.models.create_audience_group_response import CreateAudienceGroupResponse
from linebot.v3.audience.models.create_click_based_audience_group_request import CreateClickBasedAudienceGroupRequest
from linebot.v3.audience.models.create_click_based_audience_group_response import CreateClickBasedAudienceGroupResponse
from linebot.v3.audience.models.create_imp_based_audience_group_request import CreateImpBasedAudienceGroupRequest
from linebot.v3.audience.models.create_imp_based_audience_group_response import CreateImpBasedAudienceGroupResponse
from linebot.v3.audience.models.detailed_owner import DetailedOwner
from linebot.v3.audience.models.error_detail import ErrorDetail
from linebot.v3.audience.models.error_response import ErrorResponse
from linebot.v3.audience.models.get_audience_data_response import GetAudienceDataResponse
from linebot.v3.audience.models.get_audience_groups_response import GetAudienceGroupsResponse
from linebot.v3.audience.models.get_shared_audience_data_response import GetSharedAudienceDataResponse
from linebot.v3.audience.models.get_shared_audience_groups_response import GetSharedAudienceGroupsResponse
from linebot.v3.audience.models.update_audience_group_description_request import UpdateAudienceGroupDescriptionRequest



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import Field, conlist
from linebot.v3.messaging.models.demographic_filter import DemographicFilter

class OperatorDemographicFilter(DemographicFilter):
    """
    OperatorDemographicFilter
    """
    var_and: Optional[conlist(DemographicFilter)] = Field(None, alias="and")
    var_or: Optional[conlist(DemographicFilter)] = Field(None, alias="or")
    var_not: Optional[DemographicFilter] = Field(None, alias="not")
    type: str = "operator"

    __properties = ["type", "and", "or", "not"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> OperatorDemographicFilter:
        """Create an instance of OperatorDemographicFilter from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in var_and (list)
        _items = []
        if self.var_and:
            for _item in self.var_and:
                if _item:
                    _items.append(_item.to_dict())
            _dict['and'] = _items
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in var_or (list)
        _items = []
        if self.var_or:
            for _item in self.var_or:
                if _item:
                    _items.append(_item.to_dict())
            _dict['or'] = _items
        # override the default output from pydantic.v1 by calling `to_dict()` of var_not
        if self.var_not:
            _dict['not'] = self.var_not.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> OperatorDemographicFilter:
        """Create an instance of OperatorDemographicFilter from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return OperatorDemographicFilter.parse_obj(obj)

        _obj = OperatorDemographicFilter.parse_obj({
            "type": obj.get("type"),
            "var_and": [DemographicFilter.from_dict(_item) for _item in obj.get("and")] if obj.get("and") is not None else None,
            "var_or": [DemographicFilter.from_dict(_item) for _item in obj.get("or")] if obj.get("or") is not None else None,
            "var_not": DemographicFilter.from_dict(obj.get("not")) if obj.get("not") is not None else None
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class AgeDemographic(str, Enum):
    """
    AgeDemographic
    """

    """
    allowed enum values
    """
    AGE_15 = 'age_15'
    AGE_20 = 'age_20'
    AGE_25 = 'age_25'
    AGE_30 = 'age_30'
    AGE_35 = 'age_35'
    AGE_40 = 'age_40'
    AGE_45 = 'age_45'
    AGE_50 = 'age_50'
    AGE_55 = 'age_55'
    AGE_60 = 'age_60'
    AGE_65 = 'age_65'
    AGE_70 = 'age_70'

    @classmethod
    def from_json(cls, json_str: str) -> AgeDemographic:
        """Create an instance of AgeDemographic from a JSON string"""
        return AgeDemographic(json.loads(json_str))





# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.messages module."""


from abc import ABCMeta

from future.utils import with_metaclass

from linebot.models.emojis import Emojis
from .mention import Mention
from .mentionee import Mentionee
from .base import Base

from deprecated import deprecated

from linebot.deprecations import (
    LineBotSdkDeprecatedIn30
)


@deprecated(reason="Use 'from linebot.v3.webhooks import MessageContent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class Message(with_metaclass(ABCMeta, Base)):
    """Abstract Base Class of Message."""

    def __init__(self, id=None, use_raw_message=False, **kwargs):
        """__init__ method.

        :param str id: Message ID
        :param bool use_raw_message: Using original Message key as attribute
        :param kwargs:
        """
        super(Message, self).__init__(**kwargs)

        if use_raw_message:
            self.__dict__.update(kwargs)

        self.type = None
        self.id = id

    def __getitem__(self, key):
        """__getitem__ method.

        :param str key: Message key
        """
        return self.__dict__.get(key, None)


@deprecated(reason="Use 'from linebot.v3.webhooks import TextMessageContent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class TextMessage(Message):
    """TextMessage.

    https://developers.line.biz/en/reference/messaging-api/#wh-text

    Message object which contains the text sent from the source.
    """

    def __init__(self, id=None, text=None, emojis=None, mention=None, **kwargs):
        """__init__ method.

        :param str id: Message ID
        :param str text: Message text
        :param List emojis: Array of LINE emoji objects
        :param object mention: LINE mention object
        :param kwargs:
        """
        super(TextMessage, self).__init__(id=id, **kwargs)

        self.type = 'text'
        self.text = text
        if emojis:
            new_emojis = []
            for emoji in emojis:
                emoji_object = self.get_or_new_from_json_dict(
                    emoji, Emojis
                )
                if emoji_object:
                    new_emojis.append(emoji_object)
            self.emojis = new_emojis
        else:
            self.emojis = emojis

        if mention:
            mention_object = self.get_or_new_from_json_dict(
                mention, Mention
            )
            mentionees = []
            for mentionee in mention_object.mentionees:
                mentionee_object = self.get_or_new_from_json_dict(
                    mentionee, Mentionee
                )
                if mentionee_object:
                    mentionees.append(mentionee_object)
            self.mention = Mention(mentionees)
        else:
            self.mention = mention


@deprecated(reason="Use 'from linebot.v3.webhooks import ImageMessageContent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class ImageMessage(Message):
    """ImageMessage.

    https://developers.line.biz/en/reference/messaging-api/#wh-image

    Message object which contains the image content sent from the source.
    The binary image data can be retrieved with the Content API.
    """

    def __init__(self, id=None, content_provider=None, image_set=None, **kwargs):
        """__init__ method.

        :param str id: Message ID
        :param content_provider: ContentProvider object
        :type content_provider:
            :py:class:`linebot.models.messages.ContentProvider`
        :param image_set: ImageSet object
        :type image_set:
            :py:class:`linebot.models.messages.ImageSet`
        :param kwargs:
        """
        super(ImageMessage, self).__init__(id=id, **kwargs)

        self.type = 'image'
        self.content_provider = self.get_or_new_from_json_dict(
            content_provider, ContentProvider
        )
        self.image_set = self.get_or_new_from_json_dict(
            image_set, ImageSet
        )


@deprecated(reason="Use 'from linebot.v3.webhooks import VideoMessageContent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class VideoMessage(Message):
    """VideoMessage.

    https://developers.line.biz/en/reference/messaging-api/#wh-video

    Message object which contains the video content sent from the source.
    The binary video data can be retrieved with the Content API.
    """

    def __init__(self, id=None, duration=None, content_provider=None, **kwargs):
        """__init__ method.

        :param str id: Message ID
        :param long duration: Length of video file (milliseconds)
        :param content_provider: ContentProvider object
        :type content_provider:
            :py:class:`linebot.models.messages.ContentProvider`
        :param kwargs:
        """
        super(VideoMessage, self).__init__(id=id, **kwargs)

        self.type = 'video'
        self.duration = duration
        self.content_provider = self.get_or_new_from_json_dict(
            content_provider, ContentProvider
        )


@deprecated(reason="Use 'from linebot.v3.webhooks import AudioMessageContent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class AudioMessage(Message):
    """AudioMessage.

    https://developers.line.biz/en/reference/messaging-api/#wh-audio

    Message object which contains the audio content sent from the source.
    The binary audio data can be retrieved with the Content API.
    """

    def __init__(self, id=None, duration=None, content_provider=None, **kwargs):
        """__init__ method.

        :param str id: Message ID
        :param long duration: Length of audio file (milliseconds)
        :param content_provider: ContentProvider object
        :type content_provider:
            :py:class:`linebot.models.messages.ContentProvider`
        :param kwargs:
        """
        super(AudioMessage, self).__init__(id=id, **kwargs)

        self.type = 'audio'
        self.duration = duration
        self.content_provider = self.get_or_new_from_json_dict(
            content_provider, ContentProvider
        )


@deprecated(reason="Use 'from linebot.v3.webhooks import LocationMessageContent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class LocationMessage(Message):
    """LocationMessage.

    https://developers.line.biz/en/reference/messaging-api/#wh-location
    """

    def __init__(self, id=None, title=None, address=None, latitude=None, longitude=None,
                 **kwargs):
        """__init__ method.

        :param str id: Message ID
        :param str title: Title
        :param str address: Address
        :param float latitude: Latitude
        :param float longitude: Longitude
        :param kwargs:
        """
        super(LocationMessage, self).__init__(id=id, **kwargs)

        self.type = 'location'
        self.title = title
        self.address = address
        self.latitude = latitude
        self.longitude = longitude


@deprecated(reason="Use 'from linebot.v3.webhooks import StickerMessageContent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class StickerMessage(Message):
    """StickerMessage.

    https://developers.line.biz/en/reference/messaging-api/#wh-sticker

    Message object which contains the sticker data sent from the source.
    For a list of basic LINE stickers and sticker IDs, see sticker list.
    """

    def __init__(self, id=None, package_id=None, sticker_id=None,
                 sticker_resource_type=None, keywords=None, text=None, **kwargs):
        """__init__ method.

        :param str id: Message ID
        :param str package_id: Package ID
        :param str sticker_id: Sticker ID
        :param str sticker_resource_type: Sticker resource type
        :param list[str] keywords: List of up to 15 keywords describing the sticker
        :param str text: Any text entered by the user
        :param kwargs:
        """
        super(StickerMessage, self).__init__(id=id, **kwargs)

        self.type = 'sticker'
        self.package_id = package_id
        self.sticker_id = sticker_id
        self.sticker_resource_type = sticker_resource_type
        self.keywords = keywords
        self.text = text


@deprecated(reason="Use 'from linebot.v3.webhooks import FileMessageContent' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class FileMessage(Message):
    """FileMessage.

    https://developers.line.biz/en/reference/messaging-api/#wh-file

    Message object which contains the file content sent from the source.
    The binary file data can be retrieved with the Content API.
    """

    def __init__(self, id=None, file_name=None, file_size=None, **kwargs):
        """__init__ method.

        :param str id: Message ID
        :param str file_name: File Name
        :param int file_size: File Size
        :param kwargs:
        """
        super(FileMessage, self).__init__(id=id, **kwargs)

        self.type = 'file'
        self.file_size = file_size
        self.file_name = file_name


@deprecated(reason="Use 'from linebot.v3.webhooks import ContentProvider' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class ContentProvider(Base):
    """Content provider."""

    def __init__(self, type=None, original_content_url=None, preview_image_url=None, **kwargs):
        """__init__ method.

        :param str type: Provider of the content. `line` or `external`.
        :param str original_content_url: URL of the content.
        :param str preview_image_url: URL of the preview image.
        :param kwargs:
        """
        super(ContentProvider, self).__init__(**kwargs)

        self.type = type
        self.original_content_url = original_content_url
        self.preview_image_url = preview_image_url


@deprecated(reason="Use 'from linebot.v3.webhooks import ImageSet' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class ImageSet(Base):
    """Image Set."""

    def __init__(self, id=None, index=None, total=0, **kwargs):
        """__init__ method.

        :param str id: Image set ID.
        :param int index: Image number in a set of images sent simultaneously.
        :param int total: Total number of images sent simultaneously.
        :param kwargs:
        """
        super(ImageSet, self).__init__(**kwargs)

        self.id = id
        self.index = index
        self.total = total



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.v3.models.events module."""

from linebot.v3.webhooks.models.event import Event
from linebot.v3.utils import to_snake_case

class UnknownEvent(Event):
    """Unknown event.

    We welcome your contribution to line-bot-sdk-python!
    """

    @classmethod
    def new_from_json_dict(cls, data):
        """Create a new instance from a dict.

        :param data: JSON dict
        """
        new_data = {to_snake_case(key): value
                    for key, value in data.items()}

        return cls(**new_data)



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.responses module."""

from .base import Base
from .insight import (
    SubscriptionPeriodInsight, AppTypeInsight, AgeInsight,
    GenderInsight, AreaInsight, MessageInsight, ClickInsight,
    MessageStatistics, JobInsight, MessageStatisticsOfCustomAggregationUnit,
)
from .rich_menu import RichMenuSize, RichMenuArea, RichMenuAlias


class BroadcastResponse(object):
    """BroadcastResponse.

    https://developers.line.biz/en/reference/messaging-api/#send-broadcast-message
    """

    def __init__(self, request_id=None):
        """__init__ method.

        :param str request_id: Request ID. A unique ID is generated for each request
        """
        self.request_id = request_id


class ValidateReplyMessageObjectsResponse(object):
    """ValidateReplyMessageObjectsResponse.

    https://developers.line.biz/en/reference/messaging-api/#validate-message-objects-of-reply-message
    """

    def __init__(self, request_id=None):
        """__init__ method.

        :param str request_id: Request ID. A unique ID is generated for each request
        """
        self.request_id = request_id


class ValidatePushMessageObjectsResponse(object):
    """ValidatePushMessageObjectsResponse.

    https://developers.line.biz/en/reference/messaging-api/#validate-message-objects-of-push-message
    """

    def __init__(self, request_id=None):
        """__init__ method.

        :param str request_id: Request ID. A unique ID is generated for each request
        """
        self.request_id = request_id


class ValidateMulticastMessageObjectsResponse(object):
    """ValidateMulticastMessageObjectsResponse.

    https://developers.line.biz/en/reference/messaging-api/#validate-message-objects-of-multicast-message
    """

    def __init__(self, request_id=None):
        """__init__ method.

        :param str request_id: Request ID. A unique ID is generated for each request
        """
        self.request_id = request_id


class ValidateNarrowcastMessageObjectsResponse(object):
    """ValidateNarrowcastMessageObjectsResponse.

    https://developers.line.biz/en/reference/messaging-api/#validate-message-objects-of-narrowcast-message
    """

    def __init__(self, request_id=None):
        """__init__ method.

        :param str request_id: Request ID. A unique ID is generated for each request
        """
        self.request_id = request_id


class ValidateBroadcastMessageObjectsResponse(object):
    """ValidateBroadcastMessageObjectsResponse.

    https://developers.line.biz/en/reference/messaging-api/#validate-message-objects-of-broadcast-message
    """

    def __init__(self, request_id=None):
        """__init__ method.

        :param str request_id: Request ID. A unique ID is generated for each request
        """
        self.request_id = request_id


class Profile(Base):
    """Profile.

    https://developers.line.biz/en/reference/messaging-api/#get-profile
    """

    def __init__(self, display_name=None, user_id=None, picture_url=None,
                 status_message=None, language=None, **kwargs):
        """__init__ method.

        :param str display_name: Display name
        :param str user_id: User ID
        :param str picture_url: Image URL
        :param str status_message: Status message
        :param str language: Get user's language
        :param kwargs:
        """
        super(Profile, self).__init__(**kwargs)

        self.display_name = display_name
        self.user_id = user_id
        self.picture_url = picture_url
        self.status_message = status_message
        self.language = language


class Group(Base):
    """Group.

    https://developers.line.biz/en/reference/messaging-api/#get-group-id-response
    """

    def __init__(self, group_id=None, group_name=None, picture_url=None, **kwargs):
        """__init__ method.

        :param str group_id
        :param str group_name
        :param str picture_url
        :param kwargs:
        """
        super(Group, self).__init__(**kwargs)

        self.group_id = group_id
        self.group_name = group_name
        self.picture_url = picture_url


class MemberIds(Base):
    """MemberIds.

    https://developers.line.biz/en/reference/messaging-api/#get-group-member-user-ids
    https://developers.line.biz/en/reference/messaging-api/#get-room-member-user-ids
    """

    def __init__(self, member_ids=None, next=None, **kwargs):
        """__init__ method.

        :param member_ids: List of user IDs of the members in the group or room.
            Max: 100 user IDs
        :type member_ids: list[str]
        :param str next: continuationToken.
            Only returned when there are more user IDs remaining in memberIds.
        :param kwargs:
        """
        super(MemberIds, self).__init__(**kwargs)

        self.member_ids = member_ids
        self.next = next


class Content(object):
    """MessageContent.

    https://developers.line.biz/en/reference/messaging-api/#get-content
    """

    def __init__(self, response):
        """__init__ method.

        :param response: HttpResponse object
        :type response: T <= :py:class:`linebot.http_client.HttpResponse`
        """
        self.response = response

    @property
    def content_type(self):
        """Get Content-type header value.

        :rtype: str
        :return: content-type header value
        """
        return self.response.headers.get('content-type')

    @property
    def content(self):
        """Get content.

        If content size is large, should use iter_content.

        :rtype: binary
        """
        return self.response.content

    def iter_content(self, chunk_size=1024):
        """Get content as iterator (stream).

        If content size is large, should use this.

        :param chunk_size: Chunk size
        :rtype: iterator
        """
        return self.response.iter_content(chunk_size=chunk_size)


class RichMenuResponse(Base):
    """RichMenuResponse.

    https://developers.line.me/en/docs/messaging-api/reference/#rich-menu-response-object
    """

    def __init__(self, rich_menu_id=None, size=None, selected=None, name=None,
                 chat_bar_text=None, areas=None, **kwargs):
        """__init__ method.

        :param str id: Rich Menu ID
        :param size: size object which describe the rich menu displayed in the chat.
            Rich menu images must be one of the following sizes: 2500x1686, 2500x843.
        :type size: :py:class:`linebot.models.rich_menu.RichMenuSize`
        :param bool selected: true to display the rich menu by default. Otherwise, false.
        :param str name: Name of the rich menu.
            Maximum of 300 characters.
        :param str chat_bar_text: Text displayed in the chat bar.
            Maximum of 14 characters.
        :param areas: Array of area objects which define coordinates and size of tappable areas.
            Maximum of 20 area objects.
        :type areas: list[T <= :py:class:`linebot.models.rich_menu.RichMenuArea`]
        :param kwargs:
        """
        super(RichMenuResponse, self).__init__(**kwargs)

        self.rich_menu_id = rich_menu_id
        self.size = self.get_or_new_from_json_dict(size, RichMenuSize)
        self.selected = selected
        self.name = name
        self.chat_bar_text = chat_bar_text

        new_areas = []
        if areas:
            for area in areas:
                new_areas.append(
                    self.get_or_new_from_json_dict(area, RichMenuArea)
                )
        self.areas = new_areas


class RichMenuAliasResponse(Base):
    """RichMenuAliasResponse.

    https://developers.line.biz/en/reference/messaging-api/#get-rich-menu-alias-by-id-response
    """

    def __init__(self, rich_menu_alias_id=None, rich_menu_id=None, **kwargs):
        """__init__ method.

        :param str rich_menu_alias_id: Rich menu alias ID.
        :param str rich_menu_id: Rich menu ID.
        :param kwargs:
        """
        super(RichMenuAliasResponse, self).__init__(**kwargs)

        self.rich_menu_alias_id = rich_menu_alias_id
        self.rich_menu_id = rich_menu_id


class RichMenuAliasListResponse(Base):
    """RichMenuAliasListResponse.

    https://developers.line.biz/en/reference/messaging-api/#get-rich-menu-alias-list-response
    """

    def __init__(self, aliases=None, **kwargs):
        """__init__ method.

        :param aliases: Array of rich menu alias objects
        :type areas: list[T <= :py:class:`linebot.models.RichMenuAlias`]
        :param kwargs:
        """
        super(RichMenuAliasListResponse, self).__init__(**kwargs)
        new_aliases = []
        if aliases:
            for alias in aliases:
                new_aliases.append(
                    self.get_or_new_from_json_dict(alias, RichMenuAlias)
                )
        self.aliases = new_aliases


class MessageQuotaResponse(Base):
    """MessageQuotaResponse.

    https://developers.line.biz/en/reference/messaging-api/#get-quota
    """

    def __init__(self, type=None, value=None, **kwargs):
        """__init__ method.

        :param str type: Quota limitation type
        :param int value: The target limit for additional messages in the current month.
            This property is returned when the type property has a value of limited.
        :param kwargs:
        """
        super(MessageQuotaResponse, self).__init__(**kwargs)

        self.type = type
        self.value = value


class MessageQuotaConsumptionResponse(Base):
    """MessageQuotaConsumptionResponse.

    https://developers.line.biz/en/reference/messaging-api/#get-consumption
    """

    def __init__(self, total_usage=None, **kwargs):
        """__init__ method.

        :param str total_usage: The number of sent messages in the current month
        :param kwargs:
        """
        super(MessageQuotaConsumptionResponse, self).__init__(**kwargs)

        self.total_usage = total_usage


class MessageDeliveryBroadcastResponse(Base):
    """MessageDeliveryBroadcastResponse."""

    def __init__(self, status=None, success=None, **kwargs):
        """__init__ method.

        :param str status: Status of the counting process.
        :param int success: The number of messages sent with the Messaging API on the
            date specified in date.
        :param kwargs:
        """
        super(MessageDeliveryBroadcastResponse, self).__init__(**kwargs)

        self.status = status
        self.success = success


class MessageDeliveryReplyResponse(Base):
    """MessageDeliveryReplyResponse."""

    def __init__(self, status=None, success=None, **kwargs):
        """__init__ method.

        :param str status: Status of the counting process.
        :param int success: The number of messages sent with the Messaging API on the
            date specified in date.
        :param kwargs:
        """
        super(MessageDeliveryReplyResponse, self).__init__(**kwargs)

        self.status = status
        self.success = success


class MessageDeliveryPushResponse(Base):
    """MessageDeliveryPushResponse."""

    def __init__(self, status=None, success=None, **kwargs):
        """__init__ method.

        :param str status: Status of the counting process.
        :param int success: The number of messages sent with the Messaging API on the
            date specified in date.
        :param kwargs:
        """
        super(MessageDeliveryPushResponse, self).__init__(**kwargs)

        self.status = status
        self.success = success


class MessageDeliveryMulticastResponse(Base):
    """MessageDeliveryMulticastResponse."""

    def __init__(self, status=None, success=None, **kwargs):
        """__init__ method.

        :param str status: Status of the counting process.
        :param int success: The number of messages sent with the Messaging API on the
            date specified in date.
        :param kwargs:
        """
        super(MessageDeliveryMulticastResponse, self).__init__(**kwargs)

        self.status = status
        self.success = success


class MessageProgressNarrowcastResponse(Base):
    """MessageProgressNarrowcastResponse."""

    def __init__(self, phase=None, success_count=None, failure_count=None,
                 target_count=None, failed_description=None, error_code=None,
                 accepted_time=None, completed_time=None, **kwargs):
        """__init__ method.

        :param str phase: Progress status. One of `waiting`, `sending`,
            `succeeded`, or `failed`.
        :param int success_count: Number of narrowcast messages sent successful.
        :param int failure_count: Number of narrowcast messages sent failed.
        :param int target_count: Number of targeted messages sent.
        :param str failed_description: Reason why narrowcast failed, useful when
            phase is `failed`.
        :param int error_code: Summary of the error. One of `1` or `2`. `1`
            means internal error, whereas `2` indicates too few targets.
        :param str accepted_time: Narrowcast message request accepted time in milliseconds.
        :param str completed_time: Processing of narrowcast message request
            completion time in milliseconds.
        :param kwargs:
        """
        super(MessageProgressNarrowcastResponse, self).__init__(**kwargs)

        self.phase = phase
        self.success_count = success_count
        self.failure_count = failure_count
        self.target_count = target_count
        self.failed_description = failed_description
        self.error_code = error_code
        self.accepted_time = accepted_time
        self.completed_time = completed_time


class IssueLinkTokenResponse(Base):
    """IssueLinkTokenResponse.

    https://developers.line.biz/en/reference/messaging-api/#issue-link-token
    """

    def __init__(self, link_token=None, **kwargs):
        """__init__ method.

        :param str link_token: Link token.
        :param kwargs:
        """
        super(IssueLinkTokenResponse, self).__init__(**kwargs)

        self.link_token = link_token


class IssueChannelTokenResponse(Base):
    """IssueAccessTokenResponse.

    https://developers.line.biz/en/reference/messaging-api/#issue-channel-access-token
    """

    def __init__(self, access_token=None, expires_in=None, token_type=None, **kwargs):
        """__init__ method.

        :param str access_token: Short-lived channel access token.
        :param int expires_in: Time until channel access token expires in seconds
            from time the token is issued.
        :param str token_type: Bearer.
        :param kwargs:
        """
        super(IssueChannelTokenResponse, self).__init__(**kwargs)

        self.access_token = access_token
        self.expires_in = expires_in
        self.token_type = token_type


class InsightMessageDeliveryResponse(Base):
    """InsightMessageDeliveryResponse."""

    def __init__(self, status=None, broadcast=None, targeting=None, auto_response=None,
                 welcome_response=None, chat=None, api_broadcast=None, api_push=None,
                 api_multicast=None, api_reply=None, **kwargs):
        """__init__ method.

        :param str status: Calculation status. One of `ready`, `unready`, or `out_of_service`.
        :param int broadcast: Number of broadcast messages sent.
        :param int targeting: Number of targeted/segmented messages sent.
        :param int auto_response: Number of auto-response messages sent.
        :param int welcome_response: Number of greeting messages sent.
        :param int chat: Number of messages sent from LINE Official Account Manager Chat screen.
        :param int api_broadcast: Number of broadcast messages sent with
            the Send broadcast message Messaging API operation.
        :param int api_push: Number of push messages sent
            with the Send push message Messaging API operation.
        :param int api_multicast: Number of multicast messages sent with
            the Send multicast message Messaging API operation.
        :param int api_reply: Number of replies sent
            with the Send reply message Messaging API operation.
        :param int success: The number of messages sent with the Messaging API
            on the date specified in date.
        :param kwargs:
        """
        super(InsightMessageDeliveryResponse, self).__init__(**kwargs)

        self.status = status
        self.broadcast = broadcast
        self.targeting = targeting
        self.auto_response = auto_response
        self.welcome_response = welcome_response
        self.chat = chat
        self.api_broadcast = api_broadcast
        self.api_push = api_push
        self.api_multicast = api_multicast
        self.api_reply = api_reply


class InsightFollowersResponse(Base):
    """InsightFollowersResponse."""

    def __init__(self, status=None, followers=None, targeted_reaches=None, blocks=None, **kwargs):
        """__init__ method.

        :param str status: Calculation status. One of `ready`, `unready`, or `out_of_service`.
        :param int followers: The number of times, as of the specified date,
            that a user added this LINE official account as a friend for the first time.
        :param int targeted_reaches: The number of users, as of the specified date,
            that the official account can reach through targeted messages based
            on gender, age, and/or region.
        :param int blocks: The number of users blocking the account as of the specified date.
        :param kwargs:
        """
        super(InsightFollowersResponse, self).__init__(**kwargs)

        self.status = status
        self.followers = followers
        self.targeted_reaches = targeted_reaches
        self.blocks = blocks


class InsightDemographicResponse(Base):
    """InsightDemographicResponse."""

    def __init__(self, available=None, genders=None, ages=None,
                 areas=None, app_types=None, subscription_periods=None, **kwargs):
        """__init__ method.

        :param bool available: `true` if friend demographic information is available.
        :param genders: Percentage per gender.
        :type genders: list[T <= :py:class:`linebot.models.GenderInsight`]
        :param ages: Percentage per age group.
        :type ages: list[T <= :py:class:`linebot.models.AgeInsight`]
        :param areas: Percentage per area.
        :type areas: list[T <= :py:class:`linebot.models.AreaInsight`]
        :param app_types: Percentage by OS.
        :type app_types: list[T <= :py:class:`linebot.models.AppTypeInsight`]
        :param subscription_periods: Percentage per friendship duration.
        :type subscription_periods: list[T <= :py:class:`linebot.models.SubscriptionPeriodInsight`]
        :param kwargs:
        """
        super(InsightDemographicResponse, self).__init__(**kwargs)

        self.available = available
        self.genders = [self.get_or_new_from_json_dict(it, GenderInsight) for it in genders]
        self.ages = [self.get_or_new_from_json_dict(it, AgeInsight) for it in ages]
        self.areas = [self.get_or_new_from_json_dict(it, AreaInsight) for it in areas]
        self.app_types = [self.get_or_new_from_json_dict(it, AppTypeInsight) for it in app_types]
        self.subscription_periods = [self.get_or_new_from_json_dict(it, SubscriptionPeriodInsight)
                                     for it in subscription_periods]


class InsightMessageEventResponse(Base):
    """InsightMessageEventResponse."""

    def __init__(self, overview=None, messages=None, clicks=None, **kwargs):
        """__init__ method.

        :param overview: Summary of message statistics.
        :type overview: T <= :py:class:`linebot.models.MessageStatistics`
        :param messages: Array of information about individual message bubbles.
        :type messages: list[T <= :py:class:`linebot.models.MessageInsight`]
        :param clicks: Array of information about URLs in the message.
        :type clicks: list[T <= :py:class:`linebot.models.ClickInsight`]
        :param kwargs:
        """
        super(InsightMessageEventResponse, self).__init__(**kwargs)

        self.overview = self.get_or_new_from_json_dict(overview, MessageStatistics)
        self.messages = [self.get_or_new_from_json_dict(it, MessageInsight) for it in messages]
        self.clicks = [self.get_or_new_from_json_dict(it, ClickInsight) for it in clicks]


class InsightMessageEventOfCustomAggregationUnitResponse(Base):
    """InsightMessageEventResponse."""

    def __init__(self, overview=None, messages=None, clicks=None, **kwargs):
        """__init__ method.

        :param overview: Summary of message statistics.
        :type overview: T <= :py:class:`linebot.models.MessageStatisticsOfCustomAggregationUnit`
        :param messages: Array of information about individual message bubbles.
        :type messages: list[T <= :py:class:`linebot.models.MessageInsight`]
        :param clicks: Array of information about URLs in the message.
        :type clicks: list[T <= :py:class:`linebot.models.ClickInsight`]
        :param kwargs:
        """
        super(InsightMessageEventOfCustomAggregationUnitResponse, self).__init__(**kwargs)

        self.overview = self.get_or_new_from_json_dict(
            overview, MessageStatisticsOfCustomAggregationUnit)
        self.messages = [self.get_or_new_from_json_dict(it, MessageInsight) for it in messages]
        self.clicks = [self.get_or_new_from_json_dict(it, ClickInsight) for it in clicks]


class AggregationInfoResponse(Base):
    """The number of aggregation units used this month.

    https://developers.line.biz/en/reference/partner-docs/#get-number-of-units-used-this-month
    """

    def __init__(self, num_of_custom_aggregation_units=None, **kwargs):
        """__init__ method.

        :param int num_of_custom_aggregation_units: Number of aggregation units used this month.
        :param kwargs:
        """
        super(AggregationInfoResponse, self).__init__(**kwargs)

        self.num_of_custom_aggregation_units = num_of_custom_aggregation_units


class AggregationNameListResponse(Base):
    """The name list of units used this month for statistics aggregation.

    https://developers.line.biz/en/reference/partner-docs/#get-name-list-of-units-used-this-month
    """

    def __init__(self, custom_aggregation_units=None, next=None, **kwargs):
        """__init__ method.

        :param custom_aggregation_units: name list of aggregation units used this month.
            Max: 100 unit names
        :type custom_aggregation_units: list[str]
        :param str next: continuationToken.
            A continuation token to get the next array of unit names.
        :param kwargs:
        """
        super(AggregationNameListResponse, self).__init__(**kwargs)

        self.custom_aggregation_units = custom_aggregation_units
        self.next = next


class NarrowcastResponse(Base):
    """NarrowcastResponse.

    https://developers.line.biz/en/reference/messaging-api/#send-narrowcast-message
    """

    def __init__(self, request_id=None, **kwargs):
        """__init__ method.

        :param str request_id: Request ID. A unique ID is generated for each request
        :param kwargs:
        """
        super(NarrowcastResponse, self).__init__(**kwargs)

        self.request_id = request_id


class BotInfo(Base):
    """Response of `linebot.get_bot_info()` .

    https://developers.line.biz/en/reference/messaging-api/#get-bot-info
    """

    def __init__(self, user_id=None, basic_id=None, premium_id=None,
                 display_name=None, picture_url=None, chat_mode=None,
                 mark_as_read_mode=None, **kwargs):
        """__init__ method.

        :param kwargs:
        """
        super(BotInfo, self).__init__(**kwargs)

        self.user_id = user_id
        self.basic_id = basic_id
        self.premium_id = premium_id
        self.display_name = display_name
        self.picture_url = picture_url
        self.chat_mode = chat_mode
        self.mark_as_read_mode = mark_as_read_mode


class GetWebhookResponse(Base):
    """Response of `get_webhook_endpoint()` .

    https://developers.line.biz/en/reference/messaging-api/#get-webhook-endpoint-information
    """

    def __init__(self, endpoint=None, active=None, **kwargs):
        """__init__ method.

        :param str endpoint: The webhook endpoint URL.
        :param bool active: Whether the webhook is in use.
        :param kwargs:
        """
        super(GetWebhookResponse, self).__init__(**kwargs)

        self.endpoint = endpoint
        self.active = active


class TestWebhookResponse(Base):
    """Response of `test_webhook_endpoint()` .

    https://developers.line.biz/en/reference/messaging-api/#test-webhook-endpoint
    """

    def __init__(self, success=None, timestamp=None, status_code=None,
                 reason=None, detail=None, **kwargs):
        """__init__ method.

        :param bool success: Result of the communication from the LINE platform
            to the webhook URL.
        :param str timestamp: Timestamp
        :param int status_code: The HTTP status code.
        :param str reason: Reason for the response.
        :param str detail: Details of the response.
        :param kwargs:
        """
        super(TestWebhookResponse, self).__init__(**kwargs)

        self.success = success
        self.timestamp = timestamp
        self.status_code = status_code
        self.reason = reason
        self.detail = detail


class AudienceGroup(Base):
    """AudienceGroups.

    https://developers.line.biz/en/reference/messaging-api/#get-audience-group
    """

    def __init__(self, audience_group_id=None, type=None, description=None, status=None,
                 audience_count=None, created=None, is_ifa_audience=None, permission=None,
                 create_route=None, request_id=None, failed_type=None, click_url=None,
                 jobs=None, **kwargs):
        """__init__ method.

        :param int audience_group_id: The audience ID.
        :param str type: The audience type. One of `UPLOAD` or `CLICK` or `IMP` or `CHAT_TAG`
             or `FRIEND_PATH`.
        :param str description: The audience's name.
        :param str status: The audience's status. One of `IN_PROGRESS` or `READY` or `FAILED`
             or `EXPIRED`.
        :param int audience_count: The number of valid recipients.
        :param int created: When the audience was created (in UNIX time).
        :param bool is_ifa_audience: The value specified when creating an audience for uploading
            user IDs to indicate the type of accounts that will be given as recipients.
        :param str permission: Audience's update permission. Audiences linked to the same channel
            will be READ_WRITE.
        :param str create_route: How the audience was created. If omitted,
            you will get all audiences.
        :param str request_id: The request ID that was specified when the audience was created.
        :param str failed_type: The reason why the operation failed. This is only included when
            status is FAILED.  One of `AUDIENCE_GROUP_AUDIENCE_INSUFFICIENT` or `INTERNAL_ERROR`
        :param str click_url: The URL that was specified when the audience was created.
            This is only included when type is CLICK
        :param jobs: An array of jobs. This array is used to keep track of each attempt to
            add new user IDs or IFAs to an audience for uploading user IDs.
        :param kwargs:
        """
        super(AudienceGroup, self).__init__(**kwargs)

        self.audience_group_id = audience_group_id
        self.type = type
        self.description = description
        self.status = status
        self.audience_count = audience_count
        self.created = created
        self.is_ifa_audience = is_ifa_audience
        self.permission = permission
        self.create_route = create_route
        self.request_id = request_id
        self.failed_type = failed_type
        self.click_url = click_url
        if jobs:
            self.jobs = [self.get_or_new_from_json_dict(job, JobInsight) for job in jobs]


class ClickAudienceGroup(Base):
    """ClickAudienceGroup.

    https://developers.line.biz/en/reference/messaging-api/#create-click-audience-group
    """

    def __init__(self, audience_group_id=None, create_route=None, type=None, description=None,
                 created=None, permission=None, expire_timestamp=None, is_ifa_audience=None,
                 request_id=None, click_url=None, **kwargs):
        """__init__ method.

        :param int audience_group_id: The audience ID.
        :param str create_route: How the audience was created. If omitted,
            you will get all audiences.
        :param str type: The audience type. One of `UPLOAD` or `CLICK` or `IMP` or `CHAT_TAG`
             or `FRIEND_PATH`.
        :param str description: The audience's name.
        :param int created: When the audience was created (in UNIX time).
        :param str permission: Audience's update permission. Audiences linked to the same channel
            will be READ_WRITE.
        :param int expire_timestamp: Time of audience expiration. Only returned for specific
            audiences.
        :param bool is_ifa_audience: The value specified when creating an audience for uploading
            user IDs to indicate the type of accounts that will be given as recipients.
        :param str request_id: The request ID that was specified when the audience was created.
        :param str click_url: The URL that was specified when the audience was created.
            This is only included when type is CLICK
        :param kwargs:
        """
        super(ClickAudienceGroup, self).__init__(**kwargs)

        self.audience_group_id = audience_group_id
        self.create_route = create_route
        self.type = type
        self.description = description
        self.created = created
        self.permission = permission
        self.expire_timestamp = expire_timestamp
        self.is_ifa_audience = is_ifa_audience
        self.request_id = request_id
        self.click_url = click_url


class CreateAudienceGroup(Base):
    """ClickAudienceGroup.

    https://developers.line.biz/en/reference/messaging-api/#create-upload-audience-group
    """

    def __init__(self, audience_group_id=None, create_route=None, type=None, description=None,
                 created=None, permission=None, expire_timestamp=None, is_ifa_audience=None,
                 **kwargs):
        """__init__ method.

        :param int audience_group_id: The audience ID.
        :param str create_route: How the audience was created. If omitted,
            you will get all audiences.
        :param str type: The audience type. One of `UPLOAD` or `CLICK` or `IMP` or `CHAT_TAG`
             or `FRIEND_PATH`.
        :param str description: The audience's name.
        :param int created: When the audience was created (in UNIX time).
        :param str permission: Audience's update permission. Audiences linked to the same channel
            will be READ_WRITE.
        :param int expire_timestamp: Time of audience expiration. Only returned for specific
            audiences.
        :param bool is_ifa_audience: The value specified when creating an audience for uploading
            user IDs to indicate the type of accounts that will be given as recipients.
        :param kwargs:
        """
        super(CreateAudienceGroup, self).__init__(**kwargs)

        self.audience_group_id = audience_group_id
        self.create_route = create_route
        self.type = type
        self.description = description
        self.created = created
        self.permission = permission
        self.expire_timestamp = expire_timestamp
        self.is_ifa_audience = is_ifa_audience


class ImpAudienceGroup(Base):
    """ImpAudienceGroup.

    https://developers.line.biz/en/reference/messaging-api/#create-imp-audience-group
    """

    def __init__(self, audience_group_id=None, create_route=None, type=None, description=None,
                 created=None, permission=None, expire_timestamp=None, is_ifa_audience=None,
                 request_id=None, **kwargs):
        """__init__ method.

        :param int audience_group_id: The audience ID.
        :param str create_route: How the audience was created. If omitted,
            you will get all audiences.
        :param str type: The audience type. One of `UPLOAD` or `CLICK` or `IMP` or `CHAT_TAG`
             or `FRIEND_PATH`.
        :param str description: The audience's name.
        :param int created: When the audience was created (in UNIX time).
        :param str permission: Audience's update permission. Audiences linked to the same channel
            will be READ_WRITE.
        :param int expire_timestamp: Time of audience expiration. Only returned for specific
            audiences.
        :param bool is_ifa_audience: The value specified when creating an audience for uploading
            user IDs to indicate the type of accounts that will be given as recipients.
        :param str request_id: The request ID that was specified when the audience was created.
        :param kwargs:
        """
        super(ImpAudienceGroup, self).__init__(**kwargs)

        self.audience_group_id = audience_group_id
        self.create_route = create_route
        self.type = type
        self.description = description
        self.created = created
        self.permission = permission
        self.expire_timestamp = expire_timestamp
        self.is_ifa_audience = is_ifa_audience
        self.request_id = request_id


class GetAuthorityLevel(Base):
    """GetAuthorityLevel.

    https://developers.line.biz/en/reference/messaging-api/#get-authority-level
    """

    def __init__(self, authority_level=None, **kwargs):
        """__init__ method.

        :param str authority_level: The authority level for all audiences linked to a channel.
        :param kwargs:
        """
        super(GetAuthorityLevel, self).__init__(**kwargs)

        self.authority_level = authority_level


class Audience(Base):
    """Audience.

    https://developers.line.biz/en/reference/messaging-api/#update-upload-audience-group
    """

    def __init__(self, id=None, **kwargs):
        """__init__ method.

        :param str audience_id: A user ID or IFA.
        :param kwargs:
        """
        super(Audience, self).__init__(**kwargs)
        self.id = id


class UserIds(Base):
    """UserIds.

    https://developers.line.biz/en/reference/messaging-api/#get-follower-ids
    """

    def __init__(self, user_ids=None, next=None, **kwargs):
        """__init__ method.

        :param user_ids: List of user IDs of users
            that have added the LINE Official Account as a friend.
            Max: 300 user IDs
        :type user_ids: list[str]
        :param str next: continuationToken.
            A continuation token to get the next array of user IDs.
        :param kwargs:
        """
        super(UserIds, self).__init__(**kwargs)

        self.user_ids = user_ids
        self.next = next


class IssueChannelTokenResponseV2(Base):
    """IssueAccessTokenResponseV2.

    https://developers.line.biz/en/reference/messaging-api/#issue-channel-access-token-v2-1
    """

    def __init__(self, access_token=None, expires_in=None, token_type=None, key_id=None, **kwargs):
        """__init__ method.

        :param str access_token: Short-lived channel access token.
        :param int expires_in: Time until channel access token expires in seconds
            from time the token is issued.
        :param str token_type: Bearer.
        :param key_id: Unique key ID for identifying the channel access token.
        :param kwargs:
        """
        super(IssueChannelTokenResponseV2, self).__init__(**kwargs)

        self.access_token = access_token
        self.expires_in = expires_in
        self.token_type = token_type
        self.key_id = key_id


class ChannelAccessTokens(Base):
    """ChannelAccessTokens.

    https://developers.line.biz/en/reference/messaging-api/#get-issued-channel-access-tokens-v2-1
    """

    def __init__(self, access_tokens=None, **kwargs):
        """__init__ method.

        :param access_tokens: List of channel access token
        :type access_tokens: list[str]
        :param kwargs:

        """
        super(ChannelAccessTokens, self).__init__(**kwargs)

        self.access_tokens = access_tokens


class VerifyChannelTokenResponseV2(Base):
    """VerifyChannelTokenResponseV2.

    https://developers.line.biz/en/reference/messaging-api/#verfiy-channel-access-token-v2-1

    """

    def __init__(self, client_id=None, expires_in=None, scope=None, **kwargs):
        """__init__ method.

        :param str client_id: The channel ID for which the channel access token was issued.
        :param int expires_in: Number of seconds before the channel access token expires.
        :param str scope: Permissions granted to the channel access token.
        :param kwargs:

        """
        super(VerifyChannelTokenResponseV2, self).__init__(**kwargs)

        self.client_id = client_id
        self.expires_in = expires_in
        self.scope = scope


class ValidAccessTokenKeyIDsResponse(Base):
    """ValidAccessTokenKeyIDsResponse.

    https://developers.line.biz/en/reference/messaging-api/#get-all-valid-channel-access-token-key-ids-v2-1

    """

    def __init__(self, kids=None, **kwargs):
        """__init__ method.

        :param kids: Array of channel access token key IDs.
        :type kids: list[str]
        :param kwargs:
        """
        super(ValidAccessTokenKeyIDsResponse, self).__init__(**kwargs)

        self.kids = kids



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import Field, StrictStr, conlist
from linebot.v3.messaging.models.emoji import Emoji
from linebot.v3.messaging.models.message import Message
from linebot.v3.messaging.models.quick_reply import QuickReply
from linebot.v3.messaging.models.sender import Sender

class TextMessage(Message):
    """
    TextMessage
    https://developers.line.biz/en/reference/messaging-api/#text-message
    """
    text: StrictStr = Field(...)
    emojis: Optional[conlist(Emoji)] = None
    quote_token: Optional[StrictStr] = Field(None, alias="quoteToken", description="Quote token of the message you want to quote.")
    type: str = "text"

    __properties = ["type", "quickReply", "sender", "text", "emojis", "quoteToken"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TextMessage:
        """Create an instance of TextMessage from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of quick_reply
        if self.quick_reply:
            _dict['quickReply'] = self.quick_reply.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of sender
        if self.sender:
            _dict['sender'] = self.sender.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in emojis (list)
        _items = []
        if self.emojis:
            for _item in self.emojis:
                if _item:
                    _items.append(_item.to_dict())
            _dict['emojis'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TextMessage:
        """Create an instance of TextMessage from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TextMessage.parse_obj(obj)

        _obj = TextMessage.parse_obj({
            "type": obj.get("type"),
            "quick_reply": QuickReply.from_dict(obj.get("quickReply")) if obj.get("quickReply") is not None else None,
            "sender": Sender.from_dict(obj.get("sender")) if obj.get("sender") is not None else None,
            "text": obj.get("text"),
            "emojis": [Emoji.from_dict(_item) for _item in obj.get("emojis")] if obj.get("emojis") is not None else None,
            "quote_token": obj.get("quoteToken")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import unittest

import responses

from linebot import (
    LineBotApi
)


class TestLineBotApi(unittest.TestCase):
    def setUp(self):
        self.tested = LineBotApi('channel_secret')

    @responses.activate
    def test_get_group_member_ids(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/group/group_id/members/ids',
            json={
                'memberIds': ['U1', 'U2']
            },
            status=200
        )

        member_ids_response = self.tested.get_group_member_ids('group_id')

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/group/group_id/members/ids')
        self.assertEqual(member_ids_response.member_ids, ['U1', 'U2'])
        self.assertEqual(member_ids_response.next, None)

    @responses.activate
    def test_get_group_member_ids_with_start(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/group/group_id/members/ids',
            json={
                'memberIds': ['U1', 'U2'],
                'next': 'continuationToken2'
            },
            status=200
        )

        member_ids_response = self.tested.get_group_member_ids('group_id',
                                                               start='continuationToken1')

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/group/group_id/members/ids?start=continuationToken1')
        self.assertEqual(member_ids_response.member_ids, ['U1', 'U2'])
        self.assertEqual(member_ids_response.next, 'continuationToken2')

    @responses.activate
    def test_get_room_member_ids(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/room/room_id/members/ids',
            json={
                'memberIds': ['U1', 'U2']
            },
            status=200
        )

        member_ids_response = self.tested.get_room_member_ids('room_id')

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/room/room_id/members/ids')
        self.assertEqual(member_ids_response.member_ids, ['U1', 'U2'])
        self.assertEqual(member_ids_response.next, None)

    @responses.activate
    def test_get_room_member_ids_with_start(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/room/room_id/members/ids',
            json={
                'memberIds': ['U1', 'U2'],
                'next': 'continuationToken2'
            },
            status=200
        )

        member_ids_response = self.tested.get_room_member_ids('room_id',
                                                              start='continuationToken1')

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/room/room_id/members/ids?start=continuationToken1')
        self.assertEqual(member_ids_response.member_ids, ['U1', 'U2'])
        self.assertEqual(member_ids_response.next, 'continuationToken2')

    @responses.activate
    def test_get_follower_user_ids(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/followers/ids',
            json={
                'userIds': ['U1', 'U2']
            },
            status=200
        )

        member_ids_response = self.tested.get_followers_ids()

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/followers/ids?limit=300')
        self.assertEqual(member_ids_response.user_ids, ['U1', 'U2'])
        self.assertEqual(member_ids_response.next, None)

    @responses.activate
    def test_get_follower_user_ids_with_start(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/followers/ids',
            json={
                'userIds': ['U1', 'U2'],
                'next': 'continuationToken2'
            },
            status=200
        )

        member_ids_response = self.tested.get_followers_ids(start='continuationToken1')

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/followers/ids?limit=300&start=continuationToken1')
        self.assertEqual(member_ids_response.user_ids, ['U1', 'U2'])
        self.assertEqual(member_ids_response.next, 'continuationToken2')

    @responses.activate
    def test_get_follower_user_ids_with_start_and_limit(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/followers/ids',
            json={
                'userIds': ['U1', 'U2'],
                'next': 'continuationToken2'
            },
            status=200
        )

        member_ids_response = self.tested.get_followers_ids(limit=2, start='continuationToken1')

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/followers/ids?limit=2&start=continuationToken1')
        self.assertEqual(member_ids_response.user_ids, ['U1', 'U2'])
        self.assertEqual(member_ids_response.next, 'continuationToken2')


if __name__ == '__main__':
    unittest.main()



# flake8: noqa

# import apis into api package
from linebot.v3.moduleattach.api.line_module_attach import LineModuleAttach


# Async version
from linebot.v3.moduleattach.api.async_line_module_attach import AsyncLineModuleAttach




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictStr, validator

class BotInfoResponse(BaseModel):
    """
    BotInfoResponse
    https://developers.line.biz/en/reference/messaging-api/#get-bot-info
    """
    user_id: StrictStr = Field(..., alias="userId", description="Bot's user ID")
    basic_id: StrictStr = Field(..., alias="basicId", description="Bot's basic ID")
    premium_id: Optional[StrictStr] = Field(None, alias="premiumId", description="Bot's premium ID. Not included in the response if the premium ID isn't set.")
    display_name: StrictStr = Field(..., alias="displayName", description="Bot's display name")
    picture_url: Optional[StrictStr] = Field(None, alias="pictureUrl", description="Profile image URL. `https` image URL. Not included in the response if the bot doesn't have a profile image.")
    chat_mode: StrictStr = Field(..., alias="chatMode", description="Chat settings set in the LINE Official Account Manager. One of:  `chat`: Chat is set to \"On\". `bot`: Chat is set to \"Off\". ")
    mark_as_read_mode: StrictStr = Field(..., alias="markAsReadMode", description="Automatic read setting for messages. If the chat is set to \"Off\", auto is returned. If the chat is set to \"On\", manual is returned.  `auto`: Auto read setting is enabled. `manual`: Auto read setting is disabled.  ")

    __properties = ["userId", "basicId", "premiumId", "displayName", "pictureUrl", "chatMode", "markAsReadMode"]

    @validator('chat_mode')
    def chat_mode_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('chat', 'bot'):
            raise ValueError("must be one of enum values ('chat', 'bot')")
        return value

    @validator('mark_as_read_mode')
    def mark_as_read_mode_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('auto', 'manual'):
            raise ValueError("must be one of enum values ('auto', 'manual')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> BotInfoResponse:
        """Create an instance of BotInfoResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> BotInfoResponse:
        """Create an instance of BotInfoResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return BotInfoResponse.parse_obj(obj)

        _obj = BotInfoResponse.parse_obj({
            "user_id": obj.get("userId"),
            "basic_id": obj.get("basicId"),
            "premium_id": obj.get("premiumId"),
            "display_name": obj.get("displayName"),
            "picture_url": obj.get("pictureUrl"),
            "chat_mode": obj.get("chatMode"),
            "mark_as_read_mode": obj.get("markAsReadMode")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import unittest
import responses

from linebot import (
    LineBotApi
)
from linebot.models.rich_menu import RichMenuAlias


class TestLineBotApi(unittest.TestCase):
    maxDiff = None

    def setUp(self):
        self.tested = LineBotApi('channel_secret')

        self.alias_id_a = 'richmenu-alias-a'
        self.alias_id_b = 'richmenu-alias-b'
        self.rich_menu_id_a = 'richmenu-0000000'
        self.rich_menu_id_b = 'richmenu-1111111'

    @responses.activate
    def test_create_rich_menu_alias(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/alias',
            json={
                "richMenuAliasId": self.alias_id_a,
                "richMenuId": self.rich_menu_id_a
            }, status=200
        )
        alias = RichMenuAlias(
            rich_menu_alias_id=self.alias_id_a,
            rich_menu_id=self.rich_menu_id_a
        )
        result = self.tested.create_rich_menu_alias(alias)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/alias'
        )
        self.assertEqual(result, None)

    @responses.activate
    def test_delete_rich_menu(self):
        responses.add(
            responses.DELETE,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/alias/' + self.alias_id_a,
            json={}, status=200
        )

        result = self.tested.delete_rich_menu_alias(self.alias_id_a)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'DELETE')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/alias/richmenu-alias-a'
        )
        self.assertEqual(result, None)

    @responses.activate
    def test_update_rich_menu_alias(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/alias/' + self.alias_id_a,
            json={}, status=200
        )

        alias = RichMenuAlias(rich_menu_id=self.rich_menu_id_a)
        result = self.tested.update_rich_menu_alias(
            rich_menu_alias_id=self.alias_id_a, rich_menu_alias=alias
        )

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/alias/richmenu-alias-a'
        )
        self.assertEqual(result, None)

    @responses.activate
    def test_get_rich_menu_alias(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/richmenu/alias/' + self.alias_id_a,
            json={
                "richMenuAliasId": self.alias_id_a,
                "richMenuId": self.rich_menu_id_a
            }, status=200
        )

        result = self.tested.get_rich_menu_alias(rich_menu_alias_id=self.alias_id_a)
        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/alias/richmenu-alias-a'
        )
        self.assertEqual(result.rich_menu_alias_id, 'richmenu-alias-a')
        self.assertEqual(result.rich_menu_id, 'richmenu-0000000')

    @responses.activate
    def test_get_rich_menu_alias_list(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/richmenu/alias/list',
            json={
                "aliases": [{
                    "richMenuAliasId": self.alias_id_a,
                    "richMenuId": self.rich_menu_id_a
                }, {
                    "richMenuAliasId": self.alias_id_b,
                    "richMenuId": self.rich_menu_id_b
                }]
            }, status=200
        )

        result = self.tested.get_rich_menu_alias_list()
        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/alias/list'
        )
        self.assertEqual(type(result.aliases), list)
        self.assertEqual(result.aliases[0].rich_menu_alias_id, "richmenu-alias-a")
        self.assertEqual(
            result.aliases[0].rich_menu_id, "richmenu-0000000"
        )
        self.assertEqual(result.aliases[1].rich_menu_alias_id, "richmenu-alias-b")
        self.assertEqual(
            result.aliases[1].rich_menu_id, "richmenu-1111111"
        )

    @responses.activate
    def test_get_rich_menu_alias_empty_list(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/richmenu/alias/list',
            json={"aliases": []},
            status=200
        )

        result = self.tested.get_rich_menu_alias_list()
        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/richmenu/alias/list'
        )
        self.assertEqual(type(result.aliases), list)
        self.assertEqual(len(result.aliases), 0)


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictBool, StrictInt, StrictStr

class TestWebhookEndpointResponse(BaseModel):
    """
    TestWebhookEndpointResponse
    https://developers.line.biz/en/reference/messaging-api/#test-webhook-endpoint
    """
    success: Optional[StrictBool] = Field(None, description="Result of the communication from the LINE platform to the webhook URL.")
    timestamp: datetime = Field(..., description="Time of the event in milliseconds. Even in the case of a redelivered webhook, it represents the time the event occurred, not the time it was redelivered. ")
    status_code: StrictInt = Field(..., alias="statusCode", description="The HTTP status code. If the webhook response isn't received, the status code is set to zero or a negative number.")
    reason: StrictStr = Field(..., description="Reason for the response.")
    detail: StrictStr = Field(..., description="Details of the response.")

    __properties = ["success", "timestamp", "statusCode", "reason", "detail"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TestWebhookEndpointResponse:
        """Create an instance of TestWebhookEndpointResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TestWebhookEndpointResponse:
        """Create an instance of TestWebhookEndpointResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TestWebhookEndpointResponse.parse_obj(obj)

        _obj = TestWebhookEndpointResponse.parse_obj({
            "success": obj.get("success"),
            "timestamp": obj.get("timestamp"),
            "status_code": obj.get("statusCode"),
            "reason": obj.get("reason"),
            "detail": obj.get("detail")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class RichMenuBatchProgressPhase(str, Enum):
    """
    The current status. One of:  `ongoing`: Rich menu batch control is in progress. `succeeded`: Rich menu batch control is complete. `failed`: Rich menu batch control failed.           This means that the rich menu for one or more users couldn't be controlled.           There may also be users whose operations have been successfully completed. 
    """

    """
    allowed enum values
    """
    ONGOING = 'ongoing'
    SUCCEEDED = 'succeeded'
    FAILED = 'failed'

    @classmethod
    def from_json(cls, json_str: str) -> RichMenuBatchProgressPhase:
        """Create an instance of RichMenuBatchProgressPhase from a JSON string"""
        return RichMenuBatchProgressPhase(json.loads(json_str))





# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the 'License'); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an 'AS IS' BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import unittest

from linebot.models import (
    GenderFilter,
    AppTypeFilter,
    AreaFilter,
    AgeFilter,
    SubscriptionPeriodFilter
)
from tests.models.serialize_test_case import SerializeTestCase


class TestFilter(SerializeTestCase):
    def test_gender_filter(self):
        arg = {
            "one_of": ["male", "female"]
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.GENDER),
            GenderFilter(**arg).as_json_dict()
        )

    def test_app_type_filter(self):
        arg = {
            "one_of": ["ios", "android"]
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.APP_TYPE),
            AppTypeFilter(**arg).as_json_dict()
        )

    def test_age_filter(self):
        arg = {
            "gte": "age_35",
            "lt": "age_40",
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.AGE),
            AgeFilter(**arg).as_json_dict()
        )

    def test_area_filter(self):
        arg = {
            "one_of": ["jp_34", "jp_05"]
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.AREA),
            AreaFilter(**arg).as_json_dict()
        )

    def test_subscription_period_filter(self):
        arg = {
            "gte": "day_7",
            "lt": "day_30",
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.SUBSCRIPTION_PERIOD),
            SubscriptionPeriodFilter(**arg).as_json_dict()
        )


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import json
import unittest

import responses

from linebot import (
    LineBotApi
)
from linebot.models import (
    TextSendMessage
)


class TestSendTestMessage(unittest.TestCase):
    def setUp(self):
        self.tested = LineBotApi('channel_secret')
        # test data
        self.text_message = TextSendMessage(text='Hello, world')
        self.message = [{"type": "text", "text": "Hello, world"}]

    @responses.activate
    def test_push_text_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push',
            json={}, status=200
        )

        self.tested.push_message('to', self.text_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push')
        self.assertEqual(
            json.loads(request.body),
            {
                "to": "to",
                'notificationDisabled': False,
                "messages": self.message
            }
        )

    @responses.activate
    def test_push_text_message_with_retry_key(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push',
            json={}, status=200,
            headers={'X-Line-Retry-Key': '123e4567-e89b-12d3-a456-426614174000'}
        )

        self.tested.push_message(
            'to',
            self.text_message,
            retry_key='123e4567-e89b-12d3-a456-426614174000'
        )

        request = responses.calls[0].request

        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push')
        self.assertEqual(
            request.headers['X-Line-Retry-Key'],
            '123e4567-e89b-12d3-a456-426614174000'
        )
        self.assertEqual(
            json.loads(request.body),
            {
                "to": "to",
                'notificationDisabled': False,
                "messages": self.message
            }
        )

    @responses.activate
    def test_reply_text_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/reply',
            json={}, status=200
        )

        self.tested.reply_message('replyToken', self.text_message)

        request = responses.calls[0].request
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/reply')
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            json.loads(request.body),
            {
                "replyToken": "replyToken",
                'notificationDisabled': False,
                "messages": self.message
            }
        )

    @responses.activate
    def test_multicast_text_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/multicast',
            json={}, status=200
        )

        self.tested.multicast(['to1', 'to2'], self.text_message)

        request = responses.calls[0].request
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/multicast')
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            json.loads(request.body),
            {
                "to": ['to1', 'to2'],
                'notificationDisabled': False,
                "messages": self.message
            }
        )

    @responses.activate
    def test_multicast_text_message_with_retry_key(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/multicast',
            json={}, status=200,
            headers={'X-Line-Retry-Key': '123e4567-e89b-12d3-a456-426614174000'}

        )

        self.tested.multicast(
            ['to1', 'to2'],
            self.text_message,
            retry_key='123e4567-e89b-12d3-a456-426614174000'
        )

        request = responses.calls[0].request
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/multicast')
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.headers['X-Line-Retry-Key'],
            '123e4567-e89b-12d3-a456-426614174000'
        )
        self.assertEqual(
            json.loads(request.body),
            {
                "to": ['to1', 'to2'],
                'notificationDisabled': False,
                "messages": self.message
            }
        )

    @responses.activate
    def test_broadcast_text_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/broadcast',
            json={}, status=200, headers={'X-Line-Request-Id': 'request_id_test'}
        )

        response = self.tested.broadcast(self.text_message)

        request = responses.calls[0].request
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/broadcast')
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            json.loads(request.body),
            {
                'notificationDisabled': False,
                "messages": self.message
            }
        )
        self.assertEqual('request_id_test', response.request_id)

        # call with notification_disable=True
        response = self.tested.broadcast(self.text_message, notification_disabled=True)

        request = responses.calls[1].request
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/broadcast')
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            json.loads(request.body),
            {
                'notificationDisabled': True,
                "messages": self.message
            }
        )
        self.assertEqual('request_id_test', response.request_id)

    @responses.activate
    def test_broadcast_text_message_with_retry_key(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/broadcast',
            json={}, status=200,
            headers={
                'X-Line-Request-Id': 'request_id_test',
                'X-Line-Retry-Key': '123e4567-e89b-12d3-a456-426614174000'
            }
        )

        response = self.tested.broadcast(
            self.text_message,
            retry_key='123e4567-e89b-12d3-a456-426614174000'
        )

        request = responses.calls[0].request
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/broadcast')
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.headers['X-Line-Retry-Key'],
            '123e4567-e89b-12d3-a456-426614174000'
        )
        self.assertEqual(
            json.loads(request.body),
            {
                'notificationDisabled': False,
                "messages": self.message
            }
        )
        self.assertEqual('request_id_test', response.request_id)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the 'License'); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an 'AS IS' BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import unittest

from linebot.models import LinearGradientBackground
from tests.models.serialize_test_case import SerializeTestCase


class TestBackground(SerializeTestCase):
    def test_background(self):
        arg = {
            "type": "linearGradient",
            "angle": "0deg",
            "start_color": "#ff0000",
            "end_color": "#0000ff"
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.LINEAR_GRADIENT),
            LinearGradientBackground(**arg).as_json_dict()
        )


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic.v1 import BaseModel, Field, StrictStr, validator

class GetMessageContentTranscodingResponse(BaseModel):
    """
    Transcoding response
    https://developers.line.biz/en/reference/messaging-api/#verify-video-or-audio-preparation-status
    """
    status: StrictStr = Field(..., description="The preparation status. One of:  `processing`: Preparing to get content. `succeeded`: Ready to get the content. You can get the content sent by users. `failed`: Failed to prepare to get the content. ")

    __properties = ["status"]

    @validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('processing', 'succeeded', 'failed'):
            raise ValueError("must be one of enum values ('processing', 'succeeded', 'failed')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GetMessageContentTranscodingResponse:
        """Create an instance of GetMessageContentTranscodingResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> GetMessageContentTranscodingResponse:
        """Create an instance of GetMessageContentTranscodingResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return GetMessageContentTranscodingResponse.parse_obj(obj)

        _obj = GetMessageContentTranscodingResponse.parse_obj({
            "status": obj.get("status")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.emojis module."""


from abc import ABCMeta

from future.utils import with_metaclass

from .base import Base


class Background(with_metaclass(ABCMeta, Base)):
    """Background."""

    def __init__(self, **kwargs):
        """__init__ method.

        :param kwargs:
        """
        super(Background, self).__init__(**kwargs)

        self.type = None


class LinearGradientBackground(Background):
    """LinearGradientBackground."""

    def __init__(self, angle, start_color, end_color,
                 center_color=None, center_position=None, **kwargs):
        """__init__ method.

        :param str type: The type of background used
        :param str angle: The angle at which a linear gradient moves
        :param str start_color: The color at the gradient's starting point
        :param str end_color: The color at the gradient's ending point
        :param str center_color: The color in the middle of the gradient
        :param str center_position: The position of the intermediate color stop
        :param kwargs:
        """
        super(LinearGradientBackground, self).__init__(**kwargs)
        self.type = 'linearGradient'
        self.angle = angle
        self.start_color = start_color
        self.end_color = end_color
        self.center_color = center_color
        self.center_position = center_position



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.template module."""


from abc import ABCMeta

from future.utils import with_metaclass

from .actions import get_action, get_actions
from .base import Base
from .send_messages import SendMessage


class TemplateSendMessage(SendMessage):
    """TemplateSendMessage.

    https://developers.line.biz/en/reference/messaging-api/#template-messages

    Template messages are messages with predefined layouts which you can customize.
    There are three types of templates available
    that can be used to interact with users through your bot.
    """

    def __init__(self, alt_text=None, template=None, **kwargs):
        """__init__ method.

        :param str alt_text: Alternative text for unsupported devices.
        :param template: Object with the contents of the template.
        :type template: T <= :py:class:`linebot.models.template.Template`
        :param kwargs:
        """
        super(TemplateSendMessage, self).__init__(**kwargs)

        self.type = 'template'
        self.alt_text = alt_text
        self.template = self.get_or_new_from_json_dict_with_types(
            template, {
                'buttons': ButtonsTemplate,
                'confirm': ConfirmTemplate,
                'carousel': CarouselTemplate,
                'image_carousel': ImageCarouselTemplate
            }
        )


class Template(with_metaclass(ABCMeta, Base)):
    """Abstract Base Class of Template."""

    def __init__(self, **kwargs):
        """__init__ method.

        :param kwargs:
        """
        super(Template, self).__init__(**kwargs)

        self.type = None


class ButtonsTemplate(Template):
    """ButtonsTemplate.

    https://developers.line.biz/en/reference/messaging-api/#buttons

    Template message with an image, title, text, and multiple action buttons.
    """

    def __init__(self, text=None, title=None, thumbnail_image_url=None,
                 image_aspect_ratio=None,
                 image_size=None, image_background_color=None,
                 actions=None, default_action=None, **kwargs):
        """__init__ method.

        :param str text: Message text.
            Max: 160 characters (no image or title).
            Max: 60 characters (message with an image or title)
        :param str title: Title.
            Max: 40 characters
        :param str thumbnail_image_url: Image URL.
            HTTPS
            JPEG or PNG
            Aspect ratio: 1:1.51
            Max width: 1024px
            Max: 1 MB
        :param str image_aspect_ratio: Aspect ratio of the image.
            Specify one of the following values:
            rectangle: 1.51:1
            square: 1:1
        :param str image_size: Size of the image. Specify ``cover`` or ``contain``.
        :param str image_background_color: Background color of image.
            Specify a RGB color value.
        :param actions: Action when tapped.
            Max: 4
        :type actions: list[T <= :py:class:`linebot.models.actions.Action`]
        :param default_action: Action when image is tapped;
            set for the entire image, title, and text area
        :type default_action: T <= :py:class:`linebot.models.actions.Action`
        :param kwargs:
        """
        super(ButtonsTemplate, self).__init__(**kwargs)

        self.type = 'buttons'
        self.text = text
        self.title = title
        self.thumbnail_image_url = thumbnail_image_url
        self.image_aspect_ratio = image_aspect_ratio
        self.image_size = image_size
        self.image_background_color = image_background_color
        self.actions = get_actions(actions)
        self.default_action = get_action(default_action)


class ConfirmTemplate(Template):
    """ConfirmTemplate.

    https://developers.line.biz/en/reference/messaging-api/#confirm

    Template message with two action buttons.
    """

    def __init__(self, text=None, actions=None, **kwargs):
        """__init__ method.

        :param str text: Message text.
            Max: 240 characters
        :param actions: Action when tapped.
            Max: 2
        :type actions: list[T <= :py:class:`linebot.models.actions.Action`]
        :param kwargs:
        """
        super(ConfirmTemplate, self).__init__(**kwargs)

        self.type = 'confirm'
        self.text = text
        self.actions = get_actions(actions)


class CarouselTemplate(Template):
    """CarouselTemplate.

    https://developers.line.biz/en/reference/messaging-api/#carousel

    Template message with multiple columns which can be cycled like a carousel.
    """

    def __init__(self, columns=None, image_aspect_ratio=None,
                 image_size=None, **kwargs):
        """__init__ method.

        :param columns: Array of columns.
            Max: 10
        :type columns: list[T <= :py:class:`linebot.models.template.CarouselColumn`]
        :param str image_aspect_ratio: Aspect ratio of the image.
            Specify ``rectangle`` or ``square``.
        :param str image_size: Size of the image. Specify ``cover`` or ``contain``.
        :param kwargs:
        """
        super(CarouselTemplate, self).__init__(**kwargs)

        self.type = 'carousel'

        new_columns = []
        if columns:
            for column in columns:
                new_columns.append(self.get_or_new_from_json_dict(
                    column, CarouselColumn
                ))
        self.columns = new_columns
        self.image_aspect_ratio = image_aspect_ratio
        self.image_size = image_size


class ImageCarouselTemplate(Template):
    """ImageCarouselTemplate.

    https://developers.line.biz/en/reference/messaging-api/#image-carousel

    Template message with multiple images columns which can be cycled like as carousel.
    """

    def __init__(self, columns=None, **kwargs):
        """__init__ method.

        :param columns: Array of columns.
            Max: 10
        :type columns: list[T <= :py:class:`linebot.models.template.ImageCarouselColumn`]
        :param kwargs:
        """
        super(ImageCarouselTemplate, self).__init__(**kwargs)

        self.type = 'image_carousel'

        new_columns = []
        if columns:
            for column in columns:
                new_columns.append(self.get_or_new_from_json_dict(
                    column, ImageCarouselColumn
                ))
        self.columns = new_columns


class CarouselColumn(Base):
    """CarouselColumn.

    https://developers.line.biz/en/reference/messaging-api/#column-object
    """

    def __init__(self, text=None, title=None,
                 thumbnail_image_url=None, image_background_color=None,
                 actions=None, default_action=None, **kwargs):
        """__init__ method.

        :param str text: Message text.
            Max: 120 characters (no image or title)
            Max: 60 characters (message with an image or title)
        :param str title: Title.
            Max: 40 characters
        :param str thumbnail_image_url: Image URL (HTTPS). JPEG or PNG. Aspect ration is 1:1.51.
            Max width: 1024px, Max: 1 MB.
        :param str image_background_color: Background color of image.
            Specify a RGB color value.
        :param actions: Action when tapped.
            Max: 3
        :type actions: list[T <= :py:class:`linebot.models.actions.Action`]
        :param default_action: Action when image is tapped;
            set for the entire image, title, and text area
        :type default_action: T <= :py:class:`linebot.models.actions.Action`
        :param kwargs:
        """
        super(CarouselColumn, self).__init__(**kwargs)

        self.text = text
        self.title = title
        self.thumbnail_image_url = thumbnail_image_url
        self.image_background_color = image_background_color
        self.actions = get_actions(actions)
        self.default_action = get_action(default_action)


class ImageCarouselColumn(Base):
    """ImageCarouselColumn.

    https://developers.line.biz/en/reference/messaging-api/#column-object-for-image-carousel
    """

    def __init__(self, image_url=None, action=None, **kwargs):
        """__init__ method.

        :param str image_url: Image URL (HTTPS). JPEG or PNG. Aspect ration is 1:1.
            Max width: 1024px, Max: 1 MB.
        :param action: Action when image is tapped. Max: 5.
        :type action: T <= :py:class:`linebot.models.actions.Action`
        :param kwargs:
        """
        super(ImageCarouselColumn, self).__init__(**kwargs)

        self.image_url = image_url
        self.action = get_action(action)



# coding: utf-8

# flake8: noqa

"""
    Mission Stickers API

    This document describes LINE Mission Stickers API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import apis into sdk package
from linebot.v3.shop.api.shop import Shop

from linebot.v3.shop.api.async_shop import AsyncShop


# import ApiClient
from linebot.v3.shop.api_response import ApiResponse
from linebot.v3.shop.api_client import ApiClient
from linebot.v3.shop.async_api_client import AsyncApiClient
from linebot.v3.shop.configuration import Configuration
from linebot.v3.shop.exceptions import OpenApiException
from linebot.v3.shop.exceptions import ApiTypeError
from linebot.v3.shop.exceptions import ApiValueError
from linebot.v3.shop.exceptions import ApiKeyError
from linebot.v3.shop.exceptions import ApiAttributeError
from linebot.v3.shop.exceptions import ApiException

# import models into sdk package
from linebot.v3.shop.models.error_response import ErrorResponse
from linebot.v3.shop.models.mission_sticker_request import MissionStickerRequest



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.unsend module."""


from abc import ABCMeta

from future.utils import with_metaclass

from .base import Base

from deprecated import deprecated

from linebot.deprecations import (
    LineBotSdkDeprecatedIn30
)


@deprecated(reason="Use 'from linebot.v3.webhooks import UnsendDetail' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class Unsend(with_metaclass(ABCMeta, Base)):
    """Abstract Base Class of Unsend."""

    def __init__(self, message_id=None, **kwargs):
        """__init__ method.

        :param str message_id: The message ID of the unsent message.
        :param kwargs:
        """
        super(Unsend, self).__init__(**kwargs)
        self.message_id = message_id



# coding: utf-8

"""
    LIFF server API

    LIFF Server API.  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictBool

class LiffFeatures(BaseModel):
    """
    LiffFeatures
    """
    ble: Optional[StrictBool] = Field(None, description="`true` if the LIFF app supports Bluetooth® Low Energy for LINE Things. `false` otherwise. ")
    qr_code: Optional[StrictBool] = Field(False, alias="qrCode", description="`true` to use the 2D code reader in the LIFF app. false otherwise. The default value is `false`. ")

    __properties = ["ble", "qrCode"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> LiffFeatures:
        """Create an instance of LiffFeatures from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> LiffFeatures:
        """Create an instance of LiffFeatures from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return LiffFeatures.parse_obj(obj)

        _obj = LiffFeatures.parse_obj({
            "ble": obj.get("ble"),
            "qr_code": obj.get("qrCode") if obj.get("qrCode") is not None else False
        })
        return _obj

